<?php
if (!class_exists('Cryptojs')) :
/**
 * Partial Crypto-JS v1.1.0 conversion to PHP
 * Works with PHP 4 (>= 4.3) and 5
 *   Implemented: MD5, SHA-1, SHA-256, AES, Rabbit, MARC4, HMAC-MD5, HMAC-SHA1, HMMAC-SHA256, PBKDF2, Utilities
 *   Note: Rabbit currently requires 64-bit PHP (does not work correctly with 32-bit PHP)
 *   Note: SHA-256 and HMAC-SHA256 require 'hash', 'mhash', suhosin, or 3rd-party support
 *
 * Usage:
 *   $digestBytes = Cryptojs::MD5('message', array('asBytes' => true));
 *   $digestBytes = Cryptojs::SHA1('message', array('asBytes' => true));
 *   $digestBytes = Crytpojs::SHA256('message', array('asBytes' => true));
 *   $digestBytes = Cryptojs::HMAC('md5', 'message', 'key', array('asBytes' => true));
 *   $key512bit1000 = Cryptojs::PBKDF2('secret passphrase', Cryptojs_util::bytesToString(Cryptojs_util::randomBytes(16)), 64, array('iterations' => 1000));
 * 
 * Original JavaScript version:
 *   Crypto-JS v1.1.0
 *   http://code.google.com/p/crypto-js/
 *   Copyright (c) 2009, Jeff Mott. All rights reserved.
 *   http://code.google.com/p/crypto-js/wiki/License (New BSD License)
 */
class Cryptojs {

	function MD5($message, $options=null) {
		$asBytes  = (is_array($options) && isset($options['asBytes'])) ? $options['asBytes'] : false;
		$asString = (is_array($options) && isset($options['asString'])) ? $options['asString'] : false;
		
		// use built-in support
		$hex = md5($message);
		if ($asBytes || $asString) {
			$bytes = Cryptojs_util::hexToBytes($hex);
			if ($asBytes)
				return $bytes;
			else
				return Cryptojs_util::bytesToString($bytes);
		}
		else {
			return $hex;
		}
	}
	
	function SHA1($message, $options=null) {
		$asBytes  = (is_array($options) && isset($options['asBytes'])) ? $options['asBytes'] : false;
		$asString = (is_array($options) && isset($options['asString'])) ? $options['asString'] : false;
		
		// use built-in support (since PHP 4.3)
		$hex = sha1($message);
		if ($asBytes || $asString) {
			$bytes = Cryptojs_util::hexToBytes($hex);
			if ($asBytes)
				return $bytes;
			else
				return Cryptojs_util::bytesToString($bytes);
		}
		else {
			return $hex;
		}
	}
	
	function SHA256($message, $options=null) {
		$asBytes  = (is_array($options) && isset($options['asBytes'])) ? $options['asBytes'] : false;
		$asString = (is_array($options) && isset($options['asString'])) ? $options['asString'] : false;
		
		// first choice: use built-in 'hash' support (since PHP 5.1.2)
		if (function_exists('hash') && function_exists('hash_algos')) {
			$algos = hash_algos();
			if (in_array('sha256', $algos)) {
				$hex = hash('sha256', $message);
			}
		}
		
		// 2nd choice: use 'mhash' (obsoleted by 'hash')
		if (function_exists('mhash') && defined('MHASH_SHA256') && !isset($hex)) {
			$hex = bin2hex(mhash(MHASH_SHA256, $message));
		}
		
		// 3rd choice: use sha256 function
		// provided by: suhosin (http://www.hardened-php.net/suhosin/) or 3rd-party (http://nanolink.ca/pub/sha256/)
		if (function_exists('sha256') && !isset($hex)) {
			$hex = sha256($message);
		}
		
		if (isset($hex)) {
			if ($asBytes || $asString) {
				$bytes = Cryptojs_util::hexToBytes($hex);
				if ($asBytes)
					return $bytes;
				else
					return Cryptojs_util::bytesToString($bytes);
			}
			else {
				return $hex;
			}
		}
	}
	
	function HMAC($algo, $message, $key, $options=null) {
		$algo = strtolower($algo);
		if (!in_array($algo, array('md5', 'sha1', 'sha256'))) return;
	
		$asBytes  = (is_array($options) && isset($options['asBytes'])) ? $options['asBytes'] : false;
		$asString = (is_array($options) && isset($options['asString'])) ? $options['asString'] : false;

		// first choice: use built-in 'hash' support (since PHP 5.1.2)
		if (function_exists('hash_hmac') && function_exists('hash_algos')) {
			$algos = hash_algos();
			if (in_array($algo, $algos)) {
				$hex = hash_hmac($algo, $message, $key);
			}
		}
		
		$algo = strtoupper($algo);
		
		// 2nd choice: use 'mhash' (obsoleted by 'hash')
		$mhash_constant = 'MHASH_' . $algo;
		if (function_exists('mhash') && defined( $mhash_constant ) && !isset($hex)) {
			$hex = bin2hex(mhash(constant($mhash_constant), $message, $key));
		}
		
		if (isset($hex)) {
			if ($asBytes || $asString) {
				$bytes = Cryptojs_util::hexToBytes($hex);
				if ($asBytes)
					return $bytes;
				else
					return Cryptojs_util::bytesToString($bytes);
			}
			else {
				return $hex;
			}
		}
		
		// 3rd choice: use the algorithm from crypto-js
		$blocksize = Cryptojs::_blocksize($algo);
		$key = (strlen($key) > $blocksize * 4) ? Cryptojs::$algo($key, array('asBytes' => true)) : Cryptojs_util::stringToBytes($key);
		
		// XOR keys with pad constants
		$okey = $key; //array_slice($key, 0);
		$ikey = array_slice($key, 0);
		for ($i = 0; $i < $blocksize * 4; $i++) {
			// throws an error for some reason
			//$okey[$i] ^= 0x5C;
			//$ikey[$i] ^= 0x36;
			$okey[$i] = $okey[$i] ^ 0x5C;
			$ikey[$i] = $ikey[$i] ^ 0x36;
		}
		
		$bytes = Cryptojs::$algo(Cryptojs_util::bytesToString($okey) . Cryptojs::$algo(Cryptojs_util::bytesToString($ikey) . $message, array('asString' => true)), array('asBytes' => true));
		if ($asBytes)
			return $bytes;
		else if($asString)
			return Cryptojs_util::bytesToString($bytes);
		return Cryptojs_util::bytesToHex($bytes);
	}
	
	function PBKDF2($password, $salt, $keylen, $options=null) {
		// Defaults
		$hasher     = (is_array($options) && isset($options['hasher'])) ? $options['hasher'] : 'sha1';
		$iterations = (is_array($options) && isset($options['iterations'])) ? $options['iterations'] : 1;
		$asBytes    = (is_array($options) && isset($options['asBytes'])) ? $options['asBytes'] : false;
		$asString   = (is_array($options) && isset($options['asString'])) ? $options['asString'] : false;
		
		// Generate key
		$derivedKeyBytes = array();
		$blockindex = 1;
		while (count($derivedKeyBytes) < $keylen) {
			$block = Cryptojs::_PRF($hasher, $password, $salt . Cryptojs_util::bytesToString(Cryptojs_util::wordsToBytes(array($blockindex))));
			
			for ($u = $block, $i = 1; $i < $iterations; $i++) {
				$u = Cryptojs::_PRF($hasher, $password, Cryptojs_util::bytesToString($u));
				for ($j = 0; $j < count($block); $j++) {
					$block[$j] ^= $u[$j];
				}
			}
			
			foreach($block as $block_val) {
				$derivedKeyBytes[] = $block_val;
			}
			
			$blockindex++;
		}
		
		$derivedKeyBytes = array_slice($derivedKeyBytes, 0, $keylen);
		
		if ($asBytes)
			return $derivedKeyBytes;
		else if($asString)
			return Cryptojs_util::bytesToString($derivedKeyBytes);
		return Cryptojs_util::bytesToHex($derivedKeyBytes);
	}
	
	// Pseudo-random function
	function _PRF($algo, $password, $salt) {
		return Cryptojs::HMAC($algo, $salt, $password, array('asBytes' => true));
	}
	
	function _blocksize($algo) {
		$algo = strtolower($algo);
		switch($algo) {
			case 'md5':
				return 16;
			case 'sha1':
				return 16;
			case 'sha256':
				return 16;
		}
	}
}
endif;
?>