<?php
if (!class_exists('Cryptojs_util')) :
/**
 * Partial Crypto-JS v1.1.0 conversion to PHP
 * Works with PHP 4 (>= 4.3) and 5
 *   Implemented: MD5, SHA-1, SHA-256, AES, Rabbit, MARC4, HMAC-MD5, HMAC-SHA1, HMMAC-SHA256, PBKDF2, Utilities
 *   Note: Rabbit currently requires 64-bit PHP (does not work correctly with 32-bit PHP)
 *   Note: SHA-256 and HMAC-SHA256 require 'hash', 'mhash', suhosin, or 3rd-party support
 *
 * Usage:
 *   $helloBytes  = Cryptojs_util::stringToBytes('hello world');
 *   $helloString = Cryptojs_util::bytesToString($helloBytes);
 *   $helloHex    = Cryptojs_util::bytesToHex($helloBytes);
 *   $helloBytes  = Cryptojs_util::hexToBytes($helloHex);
 *   $helloBase64 = Cryptojs_util::bytesToBase64($helloBytes);
 *   $helloBytes  = Cryptojs_util::base64ToBytes($helloBase64);
 *
 * Original JavaScript version:
 *   Crypto-JS v1.1.0
 *   http://code.google.com/p/crypto-js/
 *   Copyright (c) 2009, Jeff Mott. All rights reserved.
 *   http://code.google.com/p/crypto-js/wiki/License (New BSD License)
 */
class Cryptojs_util {
	/*
	 * Unsigned right shift function, since PHP has neither >>> operator nor unsigned ints
	 *
	 * @param a  number to be shifted (32-bit integer)
	 * @param b  number of bits to shift a to the right (0..31)
	 * @return   a right-shifted and zero-filled by b bits
	 */
	function urs($a, $b) {
		$a &= 0xffffffff; $b &= 0x1f;  // (bounds check)
		if ($a&0x80000000 && $b>0) {   // if left-most bit set
			$a = ($a>>1) & 0x7fffffff;   //   right-shift one bit & clear left-most bit
			$a = $a >> ($b-1);           //   remaining right-shifts
		} else {                       // otherwise
			$a = ($a>>$b);               //   use normal right-shift
		}
		return $a;
	}
	// http://www.movable-type.co.uk/scripts/aes-php.html
	// http://pear.php.net/package/Crypt_Xtea/docs/latest/__filesource/fsource_Crypt_Xtea__Crypt_Xtea-1.1.0Xtea.php.html
	// http://www.phpbuilder.com/board/showthread.php?threadid=10366408
	// http://www.sitepoint.com/forums/showthread.php?t=449434
	// http://nanolink.ca/pub/sha256/sha256.inc.txt
	
	function base64map() {
		return 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/';
	}
	
	/**
	 * Bit-wise rotate left
	 */
	function rotl($n, $b) {
		return ($n << $b) | Cryptojs_util::urs($n, (32 - $b));
	}
	
	/**
	 * Bit-wise rotate right
	 */
	function rotr($n, $b) {
		return ($n << (32 - $b)) | Cryptojs_util::urs($n, $b);
	}
	
	/**
	 * Swap big-endian to little-endian and vice versa
	 */
	function endian($n) {
		// If number given, swap endian
		if (is_int($n)) {
			return Cryptojs_util::rotl($n, 8) & 0x00FF00FF | Cryptojs_util::rotl($n, 24) & 0xFF00FF00;
		}
		
		//Else, assume array and swap all items
		for ($i = 0; $i < count($n); $i++) {
			$n[$i] = Cryptojs_util::endian($n[$i]);
		}
		return $n;
	}
	
	/**
	 * Generate an array of any length of random bytes
	 */
	function randomBytes($n) {
		for ($bytes = array(); $n > 0; $n--) {
			$bytes[] = mt_rand(0, 255);
		}
		return $bytes;
	}
	
	/**
	 * Convert a string to a byte array
	 */
	function stringToBytes($str) {
		$bytes = array();
		for ($i = 0; $i < strlen($str); $i++) {
			$bytes[] = ord($str[$i]);
		}
		return $bytes;
	}
	
	/**
	 * Convert a byte array to a string
	 */
	function bytesToString($bytes) {
		$str = '';
		for ($i = 0; $i < count($bytes); $i++) {
			$str .= chr($bytes[$i]);
		}
		return $str;
	}
	
	/**
	 * Convert a string to big-endian 32-bit words
	 */
	function stringToWords($str) {
		$words = array();
		for ($c = 0, $b = 0; $c < strlen($str); $c++, $b += 8) {
			$words[Cryptojs_util::urs($b, 5)] |= ord($str[$c]) << (24 - $b % 32);
		}
		return $words;
	}
	
	/**
	 * Convert a byte array to big-endian 32-bits words
	 */
	function bytesToWords($bytes) {
		$words = array();
		for ($i = 0, $b = 0; $i < count($bytes); $i++, $b += 8) {
			$words[Cryptojs_util::urs($b, 5)] |= $bytes[$i] << (24 - $b % 32);
		}
		return $words;
	}
	
	/**
	 * Convert big-endian 32-bit words to a byte array
	 */
	function wordsToBytes($words) {
		$bytes = array();
		for ($b = 0; $b < count($words) * 32; $b += 8) {
			$bytes[] = Cryptojs_util::urs($words[Cryptojs_util::urs($b, 5)], (24 - $b % 32)) & 0xFF;
		}
		return $bytes;
	}
	
	/**
	 * Convert a byte array to a hex string
	 */
	function bytesToHex($bytes) {
		$hex = '';
		for ($i = 0; $i < count($bytes); $i++) {
			$temp = dechex($bytes[$i]);
			$hex .= (strlen($temp) % 2 === 0) ? $temp : '0' . $temp;
		}
		return $hex;
	}
	
	/**
	 * Convert a hex string to a byte array
	 */
	function hexToBytes($hex) {
		$bytes = array();
		for ($c = 0; $c < strlen($hex); $c += 2) {
			//$bytes[] = intval(substr($hex, $c, 2), 16);
			$bytes[] = hexdec(substr($hex, $c, 2));
		}
		return $bytes;
	}
	
	/**
	 * Convert a byte array to a base-64 string
	 */
	function bytesToBase64($bytes) {
		$base64 = '';
		$overflow = null;
		
		$base64map = Cryptojs_util::base64map();
		
		for ($i = 0; $i < count($bytes); $i++) {
			switch($i % 3) {
				case 0:
					$base64 .= $base64map[Cryptojs_util::urs($bytes[$i], 2)];
					$overflow = ($bytes[$i] & 0x3) << 4;
					break;
				case 1:
					$base64 .= $base64map[$overflow | Cryptojs_util::urs($bytes[$i], 4)];
					$overflow = ($bytes[$i] & 0xF) << 2;
					break;
				case 2:
					$base64 .= $base64map[$overflow | Cryptojs_util::urs($bytes[$i], 6)];
					$base64 .= $base64map[$bytes[$i] & 0x3F];
					$overflow = -1;
					break;
			}
		}
		
		// Encode overflow bits, if there are any
		if (!is_null($overflow) && $overflow != -1) {
			$base64 .= $base64map[$overflow];
		}
		
		// Add padding
		while (strlen($base64) % 4 != 0) $base64 .= '=';
		
		return $base64;
	}
	
	/**
	 * Convert a base-64 string to a byte array
	 */
	function base64ToBytes($base64) {
		$base64 = preg_replace('|[^A-Z0-9+/]|i', '', $base64);
		
		$bytes = array();
		$base64map = Cryptojs_util::base64map();
		
		for ($i = 0; $i < strlen($base64); $i++) {
			switch($i % 4) {
				case 1:
					$bytes[] = (strpos($base64map, $base64[$i - 1]) << 2) | Cryptojs_util::urs(strpos($base64map, $base64[$i]), 4);
					break;
				case 2:
					$bytes[] = ((strpos($base64map, $base64[$i - 1]) & 0xF) << 4) | Cryptojs_util::urs(strpos($base64map, $base64[$i]), 2);
					break;
				case 3:
					$bytes[] = ((strpos($base64map, $base64[$i - 1]) & 0x3) << 6) | (strpos($base64map, $base64[$i]));
					break;
			}
		}
		
		return $bytes;
	}
}
endif;
?>